#ifndef REGIO_REGIOBASE_H
#define REGIO_REGIOBASE_H

#include <cstddef>
#include <string>

#include <QtGlobal>

#include "regio/regiopacket.h"

#if ((__GNUC__ > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1))
#define WARN_UNUSED  __attribute__ ((warn_unused_result))
#else
#define WARN_UNUSED
#endif

namespace mlink {

    class RegIOBase {
    public:
        RegIOBase(const std::string &name = std::string()); ///< Construct M-Link Register I/O object with specified device name
        virtual ~RegIOBase();
        std::string getDevName() const {return devName;}
        virtual void reset();
        //  virtual void setLocalJtagMode(bool) = 0;
        //  virtual RegIoPacket io(const RegIoPacket &dataToSend); ///< \deprecated use ioCtrl()
        //  virtual PacketList io(const RegIoPacket &dataToSend);
        virtual void rawSend(const RegIoPacket &data) = 0; ///< \deprecated use sendPackets()
        //  virtual RegIoPacket rawRecv() = 0; ///< \deprecated use receivePackets()
        virtual void sendPackets(const PacketList &);
        virtual size_t getCtrlPacketCount() const; ///< return number of control packets received
        virtual size_t getDataSize() const; ///< return number of data words received
        PacketList getCtrlPacketBySeq(int seq, double timeout);
        virtual PacketList getCtrlPackets(size_t packetCount, double timeout) WARN_UNUSED;
        //  virtual PacketList getDataPackets(size_t count = 0, double timeout = 0) WARN_UNUSED;
        unsigned long getDriverVersion() const;
        unsigned long getDriverRevision() const;
        virtual unsigned long getMaxDataSize() const = 0;

//        qint64 read(char * data, qint64 maxBytes); ///< read data packet contents
        void receive();
        double getPollInterval() const;
        RawPacketList popDataPackets() { RawPacketList pl = rxBufData; rxBufData.clear(); return pl; }

    protected:
        virtual double getDeviceThroughput() const;
        virtual qint64 getRxBufSizeLimit() const;
        void setDevName(const std::string &name) {devName = name;}
        virtual RawPacketList getRxPackets(double timeout) = 0; ///< get packets from hardware
        void demuxRxBuf(const RawPacketList&);
        bool packetTypeIsData(int type);

        std::string devName;
        unsigned long driverVersion;
        unsigned long driverRevision;

        int bufSizeLimit;
        PacketList rxBufCtrl;
//        PacketRawData rxBufData;
        RawPacketList rxBufData;
    };

    double getCurrentTimestamp();

};

#endif /* REGIO_REGIOBASE_H */
