#ifndef REGIOEMU_H
#define REGIOEMU_H

#include <vector>
#include <map>
#include <list>
#include <QSharedPointer>
#include "regio/regiobase.h"

namespace mlink {

    class RegIOEmuBase: public RegIOBase
    {
    public:
        RegIOEmuBase(const std::string &name = std::string());
        virtual ~RegIOEmuBase();
        static std::string defaultDevName();
        virtual void reset();
        bool serdes_loopback_test(unsigned int count = 10000); // internal loopback in serdes
        bool link_loopback_test(unsigned int count = 10000); // remote loopback
        //  virtual RegIoPacket io(const RegIoPacket &dataToSend);
        virtual void rawSend(const RegIoPacket &data);
        virtual RegIoPacket rawRecv();
        //  virtual void setLocalJtagMode(bool) { }
        //  virtual PacketList getData();
        virtual unsigned long getMaxDataSize() const { return 1000; }

    protected:
        virtual RawPacketList getRxPackets(double timeout);
        virtual void processPackets() = 0;

        PacketList rxBuf, txBuf;
    };

    class DeviceEmu;

    class RegIOEmu: public RegIOEmuBase {
    public:
        void insertDeviceEmu(int address, QSharedPointer<DeviceEmu> p) { devices[address] = p; }
    protected:
        virtual void reset();
        virtual void processPackets();
        typedef QSharedPointer<DeviceEmu> DeviceEmuPtr;
        typedef std::map<int,  DeviceEmuPtr> DeviceList;
        DeviceList devices;
    };

};

#endif /* REGIOEMU_H*/
